#!/usr/bin/env python

# Copyright (c) 2020, 2025 Oracle and/or its affiliates.
# Licensed under the Universal Permissive License v 1.0 as shown at https://oss.oracle.com/licenses/upl/

import contextlib
import inspect
import os
from typing import Dict, Optional

from ads.common.config import DEFAULT_CONFIG_PATH, DEFAULT_CONFIG_PROFILE, Config, Mode

OCI_ODSC_SERVICE_ENDPOINT = os.environ.get("OCI_ODSC_SERVICE_ENDPOINT")
OCI_IDENTITY_SERVICE_ENDPOINT = os.environ.get("OCI_IDENTITY_SERVICE_ENDPOINT")
NB_SESSION_COMPARTMENT_OCID = os.environ.get("NB_SESSION_COMPARTMENT_OCID")
PROJECT_OCID = os.environ.get("PROJECT_OCID") or os.environ.get("PIPELINE_PROJECT_OCID")
NB_SESSION_OCID = os.environ.get("NB_SESSION_OCID")
USER_OCID = os.environ.get("USER_OCID")
OCI_RESOURCE_PRINCIPAL_VERSION = os.environ.get("OCI_RESOURCE_PRINCIPAL_VERSION")
OCI_RESOURCE_PRINCIPAL_RPT_PATH = os.environ.get("OCI_RESOURCE_PRINCIPAL_RPT_PATH")
OCI_RESOURCE_PRINCIPAL_RPT_ID = os.environ.get("OCI_RESOURCE_PRINCIPAL_RPT_ID")
TENANCY_OCID = os.environ.get("TENANCY_OCID")
OCI_REGION_METADATA = os.environ.get("OCI_REGION_METADATA")
JOB_RUN_OCID = os.environ.get("JOB_RUN_OCID")
JOB_RUN_COMPARTMENT_OCID = os.environ.get("JOB_RUN_COMPARTMENT_OCID")
PIPELINE_RUN_OCID = os.environ.get("PIPELINE_RUN_OCID")
PIPELINE_RUN_COMPARTMENT_OCID = os.environ.get("PIPELINE_RUN_COMPARTMENT_OCID")
PIPELINE_COMPARTMENT_OCID = os.environ.get("PIPELINE_COMPARTMENT_OCID")

CONDA_BUCKET_NAME = os.environ.get("CONDA_BUCKET_NAME", "service-conda-packs")
CONDA_BUCKET_NS = os.environ.get("CONDA_BUCKET_NS", "id19sfcrra6z")
OCI_RESOURCE_PRINCIPAL_RPT_ENDPOINT = os.environ.get(
    "OCI_RESOURCE_PRINCIPAL_RPT_ENDPOINT"
)
PROJECT_COMPARTMENT_OCID = os.environ.get("PROJECT_COMPARTMENT_OCID")
COMPARTMENT_OCID = (
    NB_SESSION_COMPARTMENT_OCID
    or PROJECT_COMPARTMENT_OCID
    or JOB_RUN_COMPARTMENT_OCID
    or PIPELINE_RUN_COMPARTMENT_OCID
)
MD_OCID = os.environ.get("MD_OCID")
DATAFLOW_RUN_OCID = os.environ.get("DATAFLOW_RUN_ID")
RESOURCE_OCID = (
    NB_SESSION_OCID or JOB_RUN_OCID or MD_OCID or PIPELINE_RUN_OCID or DATAFLOW_RUN_OCID
)
NO_CONTAINER = os.environ.get("NO_CONTAINER")
TMPDIR = os.environ.get("TMPDIR")

AQUA_MODEL_DEPLOYMENT_CONFIG = os.environ.get(
    "AQUA_DEPLOYMENT_CONFIG", "deployment_config.json"
)
AQUA_MODEL_FINETUNING_CONFIG = os.environ.get(
    "AQUA_MODEL_FINETUNING_CONFIG", "ft_config.json"
)
AQUA_CONTAINER_INDEX_CONFIG = os.environ.get(
    "AQUA_CONTAINER_INDEX_CONFIG", "container_index.json"
)
AQUA_SERVICE_MODELS = "aqua-service-models-compartment"
AQUA_DEPLOYMENT_CONTAINER_METADATA_NAME = "deployment-container"
AQUA_FINETUNING_CONTAINER_METADATA_NAME = "finetune-container"
AQUA_EVALUATION_CONTAINER_METADATA_NAME = "evaluation-container"
AQUA_DEPLOYMENT_CONTAINER_CMD_VAR_METADATA_NAME = "container-cmd-var"
AQUA_DEPLOYMENT_CONTAINER_URI_METADATA_NAME = "deployment-container-uri"
AQUA_DEPLOYMENT_CONTAINER_OVERRIDE_FLAG_METADATA_NAME = "deployment-container-custom"
AQUA_FINETUNING_CONTAINER_OVERRIDE_FLAG_METADATA_NAME = "finetune-container-custom"
AQUA_MODEL_DEPLOYMENT_FOLDER = "/opt/ds/model/deployed_model/"
AQUA_SERVED_MODEL_NAME = "odsc-llm"
AQUA_CONFIG_FOLDER = os.path.join(
    os.path.dirname(os.path.realpath(__file__)), "aqua/config/"
)
AQUA_JOB_SUBNET_ID = os.environ.get("AQUA_JOB_SUBNET_ID", None)
AQUA_SERVICE_MODELS_BUCKET = os.environ.get(
    "AQUA_SERVICE_MODELS_BUCKET", "service-managed-models"
)
AQUA_TELEMETRY_BUCKET = os.environ.get(
    "AQUA_TELEMETRY_BUCKET", "service-managed-models"
)
AQUA_TELEMETRY_BUCKET_NS = os.environ.get("AQUA_TELEMETRY_BUCKET_NS", CONDA_BUCKET_NS)

DEBUG_TELEMETRY = os.environ.get("DEBUG_TELEMETRY", None)
AQUA_SERVICE_NAME = "aqua"
DATA_SCIENCE_SERVICE_NAME = "data-science"
USER = "USER"
SERVICE = "SERVICE"


THREADED_DEFAULT_TIMEOUT = 5
try:
    THREADED_DEFAULT_TIMEOUT = int(
        os.environ.get("THREADED_DEFAULT_TIMEOUT", THREADED_DEFAULT_TIMEOUT)
    )
except ValueError:
    pass


def export(
    uri: Optional[str] = DEFAULT_CONFIG_PATH,
    auth: Dict = None,
    force_overwrite: Optional[bool] = False,
) -> Config:
    """Exports the ADS config.

    Parameters
    ----------
    uri: (str, optional). Defaults to `~/.ads/config`.
        The path to the config file. Can be local or Object Storage file.
    auth: (Dict, optional). Defaults to None.
        The default authentication is set using `ads.set_auth` API. If you need to override the
        default, use the `ads.common.auth.api_keys` or `ads.common.auth.resource_principal` to create appropriate
        authentication signer and kwargs required to instantiate IdentityClient object.
    force_overwrite: (bool, optional). Defaults to `False`.
        Overwrites the config if exists.

    Returns
    -------
    ads.Config
        The ADS config object.
    """
    return Config().load().save(uri=uri, auth=auth, force_overwrite=force_overwrite)


def load(
    uri: Optional[str] = DEFAULT_CONFIG_PATH,
    auth: Dict = None,
    force_overwrite: Optional[bool] = False,
) -> Config:
    """Imports the ADS config.

    The config will be imported from the URI and saved the `~/.ads/config`.

    Parameters
    ----------
    uri: (str, optional). Defaults to `~/.ads/config`.
        The path where the config file needs to be saved. Can be local or Object Storage file.
    auth: (Dict, optional). Defaults to None.
        The default authentication is set using `ads.set_auth` API. If you need to override the
        default, use the `ads.common.auth.api_keys` or `ads.common.auth.resource_principal` to create appropriate
        authentication signer and kwargs required to instantiate IdentityClient object.
    force_overwrite: (bool, optional). Defaults to `False`.
            Overwrites the config if exists.

    Returns
    -------
    ads.Config
        The ADS config object.
    """
    return Config().load(uri=uri, auth=auth).save(force_overwrite=force_overwrite)


@contextlib.contextmanager
def open(
    uri: Optional[str] = DEFAULT_CONFIG_PATH,
    profile: Optional[str] = DEFAULT_CONFIG_PROFILE,
    mode: Optional[str] = Mode.READ,
    auth: Dict = None,
):
    """Context manager helping to read and write config files.

    Parameters
    ----------
    uri: (str, optional). Defaults to `~/.ads/config`.
        The path to the config file. Can be local or Object Storage file.
    profile: (str, optional). Defaults to `DEFAULT`
        The name of the profile to be loaded.
    mode: (str, optional). Defaults to `r`.
        The config mode. Supported values: ['r', 'w']
    auth: (Dict, optional). Defaults to None.
        The default authentication is set using `ads.set_auth` API. If you need to override the
        default, use the `ads.common.auth.api_keys` or `ads.common.auth.resource_principal` to create appropriate
        authentication signer and kwargs required to instantiate IdentityClient object.

    Yields
    ------
    ConfigSection
        The config section object.
    """
    section = profile.upper()
    config = Config(uri=uri, auth=auth)

    try:
        config.load()
    except:
        if mode == Mode.READ:
            raise

    if not config.section_exists(section):
        config[section] = {}

    defined_globals = {}
    section_obj = config[section]
    section_keys = section_obj.keys()
    frame = inspect.currentframe().f_back.f_back

    # Saves original globals and adding new ones
    for key in section_keys:
        if key in frame.f_globals:
            defined_globals[key] = frame.f_globals[key]
        frame.f_globals[key] = section_obj[key]

    frame.f_globals["config"] = section_obj

    try:
        yield section_obj
    finally:
        # Removes config attributes from the globals
        for key in section_keys:
            frame.f_globals.pop(key, None)

        frame.f_globals.pop("config", None)

        # Restores original globals
        for key in defined_globals:
            frame.f_globals[key] = defined_globals[key]

        # Saving config if it necessary
        if mode == Mode.WRITE:
            config.save(force_overwrite=True)



#!/usr/bin/env python
# Copyright (c) 2021, 2025 Oracle and/or its affiliates.
# Licensed under the Universal Permissive License v 1.0 as shown at https://oss.oracle.com/licenses/upl/

import json
import logging
import sys
import traceback
import uuid

import fire
from pydantic import BaseModel

from ads.common import logger

try:
    import click

    import ads.jobs.cli
    import ads.opctl.cli
    import ads.opctl.operator.cli
    import ads.pipeline.cli
except Exception as ex:
    print(
        "Please run `pip install oracle-ads[opctl]` to install "
        "the required dependencies for ADS CLI. \n"
        f"{str(ex)}"
    )
    logger.debug(ex)
    logger.debug(traceback.format_exc())
    sys.exit()

# https://packaging.python.org/en/latest/guides/single-sourcing-package-version/#single-sourcing-the-package-version
if sys.version_info >= (3, 8):
    from importlib import metadata
else:
    import importlib_metadata as metadata


ADS_VERSION = metadata.version("oracle_ads")


@click.group()
@click.version_option(version=ADS_VERSION, prog_name="ads")
@click.help_option("--help", "-h")
def click_cli():
    pass


@click.command
def aqua_cli():
    """CLI for AQUA."""
    # This is a dummy entry for click.
    # The `ads aqua` commands are handled by AquaCommand


click_cli.add_command(ads.opctl.cli.commands)
click_cli.add_command(ads.jobs.cli.commands)
click_cli.add_command(ads.pipeline.cli.commands)
click_cli.add_command(ads.opctl.operator.cli.commands)
click_cli.add_command(aqua_cli, name="aqua")


# fix for fire issue with --help
# https://github.com/google/python-fire/issues/258
def _SeparateFlagArgs(args):
    try:
        index = args.index("--help")
        args = args[:index]
        return args, ["--help"]
    except ValueError:
        return args, []


fire.core.parser.SeparateFlagArgs = _SeparateFlagArgs


def serialize(data):
    """Serialize dataclass objects or lists of dataclass objects.
    Parameters:
        data: A dataclass object or a list of dataclass objects.
    Returns:
        None
    Prints:
        The string representation of each dataclass object.
    """
    if isinstance(data, list):
        for item in data:
            if isinstance(item, BaseModel):
                print(json.dumps(item.dict(), indent=4))
            else:
                print(str(item))
    elif isinstance(data, BaseModel):
        print(json.dumps(data.dict(), indent=4))
    else:
        print(str(data))


def exit_program(ex: Exception, logger: "logging.Logger") -> None:
    """
    Logs the exception and exits the program with a specific exit code.

    This function logs the full traceback and the exception message, then terminates
    the program with an exit code. If the exception object has an 'exit_code' attribute,
    it uses that as the exit code; otherwise, it defaults to 1.

    Parameters
    ----------
    ex (Exception):
        The exception that triggered the program exit. This exception
        should ideally contain an 'exit_code' attribute, but it is not mandatory.
    logger (Logger):
        A logging.Logger instance used to log the traceback and the error message.

    Returns
    -------
    None:
        This function does not return anything because it calls sys.exit,
        terminating the process.

    Examples
    --------

    >>> import logging
    >>> logger = logging.getLogger('ExampleLogger')
    >>> try:
    ...     raise ValueError("An error occurred")
    ... except Exception as e:
    ...     exit_program(e, logger)
    """

    request_id = str(uuid.uuid4())
    logger.debug(f"Error Request ID: {request_id}\nError: {traceback.format_exc()}")
    logger.error(f"Error Request ID: {request_id}\n" f"Error: {str(ex)}")

    exit_code = getattr(ex, "exit_code", 1)
    logger.error(f"Exit code: {exit_code}")
    sys.exit(exit_code)


def cli():
    if len(sys.argv) > 1 and sys.argv[1] == "aqua":
        from ads.aqua import logger as aqua_logger
        from ads.aqua.cli import AquaCommand

        try:
            fire.Fire(
                AquaCommand, command=sys.argv[2:], name="ads aqua", serialize=serialize
            )
        except Exception as err:
            exit_program(err, aqua_logger)
    else:
        click_cli()


if __name__ == "__main__":
    cli()
